using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Programs;
using SlatedGameToolkit.Framework.Graphics.Shaders;

namespace SlatedGameToolkit.Framework.Graphics
{
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLClearColour(float r, float g, float b);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLClear(GLEnum flags);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLViewport(int x, int y, int width, int height);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate UIntPtr GLCreateShader(GLEnum type);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate UIntPtr GLShaderSource(UIntPtr shaderHandle, uint count, string[] program, int[] length);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLCompileShader(UIntPtr shaderHandle);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGetShaderInfoLog(UIntPtr shader, uint maxLength, out uint writtenLength, byte[] output);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate UIntPtr GLCreateProgram();
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate UIntPtr GLDeleteProgram(UIntPtr program);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLAttachShader(UIntPtr program, UIntPtr shader);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDetachShader(UIntPtr program, UIntPtr shader);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDeleteShader(UIntPtr shader);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLBindAttribLocation(UIntPtr program, uint index, string name);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGetProgramInfoLog(UIntPtr program, uint maxLength, out uint writtenLength, byte[] output);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLLinkProgram(UIntPtr program);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLProgramParameter(UIntPtr program, GLEnum parameterName, int value);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLUseProgram(UIntPtr program);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGenProgramPipelines(uint size, UIntPtr[] pipelines);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDeleteProgramPipelines(uint size, UIntPtr[] pipelines);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLBindProgramPipeline(UIntPtr pipeline);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLUseProgramStages(UIntPtr pipeline, GLEnum bitField, UIntPtr program);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate uint GLGetError();
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate int GLGetAttribLocation(UIntPtr program, string attribName);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLBindBuffer(GLEnum target, UIntPtr buffer);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGenBuffers(uint size, UIntPtr[] buffers);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public unsafe delegate void GLBufferData(GLEnum target, uint size, Array data, GLEnum usage);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDeleteBuffers(uint size, UIntPtr[] buffers);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLVertexAttribPointer(uint index, int size, GLEnum type, bool normalized, uint stride, uint offset);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGenVertexArrays(uint amount, UIntPtr[] arrays);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDeleteVertexArrays(uint amount, UIntPtr[] arrays);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLBindVertexArray(UIntPtr array);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]   
    public delegate void GLEnableVertexAttribArray(uint attribIndex);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDisableVertexAttribArray(uint attribIndex);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDrawArrays(GLEnum mode, int first, uint count);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDrawElements(GLEnum mode, uint count, int offset);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLMultiDrawArrays(GLEnum mode, int[] first, uint[] count, uint primout);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLMultiDrawElements(GLEnum mode, uint[] count, GLEnum type, uint[] indiceOffsets, int length);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGenTextures(uint n, UIntPtr[] textureHandles);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLBindTexture(GLEnum target, UIntPtr texture);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLDeleteTextures(uint n, UIntPtr[] textureHandles);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLTexParameteri(GLEnum target, GLEnum pname, int value);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLTexParameterf(GLEnum target, GLEnum pname, float value);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLTexParameteriv(GLEnum target, GLEnum pname, int[] values);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLTexParameterfv(GLEnum target, GLEnum pname, float[] values);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLTexImage2D(GLEnum target, int level, int publicFormat, uint width, uint height, int border, GLEnum format, GLEnum type, byte[] data);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLGenerateMipmap(GLEnum target);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate int GLGetUniformLocation(UIntPtr program, string name);
    [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
    public delegate void GLUniformMatrix4fv(int index, uint count, bool transpose, float[] matrix);

    public static class OpenGL {
        public const int OpenGLMajorVersion = 3;
        public const int OpenGLMinorVersion = 3;
        /// <summary>
        /// Attempts to retrieve and instantiate an OpenGL function as a C# delegate.
        /// Make sure the delegates signature is correct as failing to do so will potentially 
        /// lead to a CLR error on usage.
        /// 
        /// Uses SDL2 as the function address retriever.
        /// </summary>
        /// <param name="functionName">The process name, case sensitive, exactly as OpenGL defines it.</param>
        /// <typeparam name="T">The delegate type to cast to.</typeparam>
        /// <returns>The delegate associated with the retrieved path.</returns>
        public static T RetrieveGLDelegate<T>(string functionName) where T : Delegate {
            GameEngine.Logger.Debug(String.Format("Retrieving function with name: {0}", functionName));
            IntPtr functionAddress = SDL.SDL_GL_GetProcAddress(functionName);
            if (functionAddress.Equals(IntPtr.Zero)) throw new FrameworkSDLException();
            return Marshal.GetDelegateForFunctionPointer<T>(functionAddress);
        }
    }
}