using System;
using System.Text;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics;
using SlatedGameToolkit.Framework.Graphics.Shaders;

namespace SlatedGameToolkit.Framework.Graphics.Programs
{
    public class GLShaderProgram : GLProgram
    {
        private readonly GLShader[] shaders;
        private readonly GLAttachShader attachShader;
        private readonly GLDetachShader detachShader;
        public GLShaderProgram(params GLShader[] shaders) : base() {
            if (shaders.Length == 0) throw new ArgumentException("Requires at least one shader for shader program.");
            this.shaders = shaders;

            attachShader = OpenGL.RetrieveGLDelegate<GLAttachShader>("glAttachShader");
            detachShader = OpenGL.RetrieveGLDelegate<GLDetachShader>("glDetachShader");

            
            foreach (GLShader shader in shaders)
            {
                attachShader(handle, shader.Handle);
                OpenGLErrorException.CheckGLErrorStatus();
            }

            linkProgram(handle);
            uint length;
            byte[] log = new byte[2048];
            getProgramInfoLog(handle, (uint)log.Length, out length, log);
            if (length > 0) {
                Dispose();
                throw new OpenGLException(Encoding.UTF8.GetString(log));
            }
            OpenGLErrorException.CheckGLErrorStatus();

            foreach (GLShader shader in shaders)
            {
                detachShader(handle, shader.Handle);
                OpenGLErrorException.CheckGLErrorStatus();
            }
        }


        /// <summary>
        /// Disposes of the shaders and this program.
        /// </summary>
        public override void Dispose() {
            foreach (GLShader shader in shaders)
            {
                shader.Dispose();
            }
            base.Dispose();
        }

        ~GLShaderProgram() {
            Dispose();
        }
    }
}