using System.Numerics;

namespace SlatedGameToolkit.Framework.Graphics.Render
{
    public class Camera {
        private bool viewUpdated, projectionUpdated;
        public virtual Vector3 Up { get; set; }
        private Vector3 trans, lookAt;
        private float nearField, farField;
        private float width, height;
        private Matrix4x4 lookAtMatrix;
        private Matrix4x4 view, projection;
        public virtual bool Orthographic { get; set; }

        public virtual Vector3 Position {
            get {
                return trans;
            }

            set {
                this.trans = value;
                viewUpdated = true;
            }
        }

        public Vector3 LookAt {
            get {
                return lookAt;
            }

            set {
                this.lookAt = value;
                lookAtMatrix = Matrix4x4.CreateLookAt(-trans, value, Up);
                viewUpdated = true;
            }
        }

        public virtual float NearField {
            get {
                return nearField;
            }

            set {
                nearField = value;
                projectionUpdated = true;
            }
        }

        public virtual float FarField {
            get {
                return farField;
            }

            set {
                farField = value;
                projectionUpdated = true;
            }
        }

        public virtual float Width {
            get {
                return width;
            }
             set {
                 width = value;
                 projectionUpdated = true;
             }
        }
        public virtual float Height {
            get {
                return height;
            }
            set {
                height = value;
                projectionUpdated = true;
            }
        }

        public Matrix4x4 ViewMatrix {
            get {
                if (viewUpdated) {
                    view = Matrix4x4.CreateTranslation(-trans) * lookAtMatrix;
                    viewUpdated = false;
                }
                return view;
            }
        }

        public Matrix4x4 ProjectionMatrix {
            get {
                if (projectionUpdated) {
                    if (Orthographic) {
                        projection = Matrix4x4.CreateOrthographic(width, height, nearField, farField);
                    } else {
                        projection = Matrix4x4.CreatePerspective(width, height, nearField, farField);
                    }
                    projectionUpdated = false;
                }
                return projection;
            }
        }
    }
}