using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics;
using SlatedGameToolkit.Framework.Graphics.Programs;
using SlatedGameToolkit.Framework.Graphics.Window;

namespace SlatedGameToolkit.Framework.Graphics.Shaders
{
    public abstract class GLShader : IDisposable {
        private WindowContext context;
        public virtual UIntPtr Handle { get; protected set; }

        protected GLCreateShader createShader;
        protected GLShaderSource shaderSource;
        protected GLCompileShader compileShader;
        protected GLGetShaderInfoLog getShaderInfoLog;
        private GLDeleteShader deleteShader;

        public GLShader() {
            context = WindowContextsManager.CurrentWindowContext;
            createShader = OpenGL.RetrieveGLDelegate<GLCreateShader>("glCreateShader");
            shaderSource = OpenGL.RetrieveGLDelegate<GLShaderSource>("glShaderSource");
            compileShader = OpenGL.RetrieveGLDelegate<GLCompileShader>("glCompileShader");
            getShaderInfoLog = OpenGL.RetrieveGLDelegate<GLGetShaderInfoLog>("glGetShaderInfoLog");
            deleteShader = OpenGL.RetrieveGLDelegate<GLDeleteShader>("glDeleteShader");
        }

        
        /// <summary>
        /// Disposes of the shader at the handle.
        /// </summary>
        public virtual void Dispose()
        {
            deleteShader(Handle);
            OpenGLErrorException.CheckGLErrorStatus();
        }

        ~GLShader() {
            Dispose();
        }
    }
}