using System;
using System.Runtime.InteropServices;
using SDL2;
using SlatedGameToolkit.Framework.Exceptions;
using SlatedGameToolkit.Framework.Graphics.Textures;
using SlatedGameToolkit.Framework.Loader;

namespace SlatedGameToolkit.Framework.Loader
{
    public class TextureLoader : ILoadable<Texture>
    {
        public string FileName {get; private set; }
        public TextureLoader(string path) {
            this.FileName = path;
        }

        public Texture Load(PathModifier pathModifier = null)
        {
            IntPtr ptr = SDL_image.IMG_Load(pathModifier == null ? FileName : pathModifier(FileName));
            if (ptr.Equals(IntPtr.Zero)) throw new FrameworkSDLException();
            SDL.SDL_Surface surface = Marshal.PtrToStructure<SDL.SDL_Surface>(ptr);
            TextureData textureData = new TextureData();
            textureData.data = new byte[surface.pitch * surface.h];
            Marshal.Copy(surface.pixels, textureData.data, 0, textureData.data.Length);
            Texture useable = new Texture(textureData);
            return useable;
        }
    }
}